@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2025 Phosh e.V.
+ * Copyright (C) 2025 Phosh.mobi e.V.
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -98,10 +98,10 @@ pos_emoji_db_get_data (PosEmojiDb *self)
 }
 
 /**
- * pos_emoji_match_by_name:
+ * pos_emoji_db_match_by_name:
  * @self: The emoji db
- * @name: The emoji name (or prefix) to match on
- * @max_mathces: The maximum number of matches. `0` means "all matches".
+ * @match: The emoji name (or prefix) to match on
+ * @max_matches: The maximum number of matches. `0` means "all matches".
  *
  * Returns: (transfer full): The matched emojis
  */
@@ -134,7 +134,7 @@ pos_emoji_db_match_by_name (PosEmojiDb *self, const char *match, guint max_match
 
         g_variant_get_child (codes, i, "u", &code);
         if (code)
-          p += g_unichar_to_utf8 (GINT32_FROM_LE (code), p);
+          p += g_unichar_to_utf8 (code, p);
       }
       p += g_unichar_to_utf8 (0xFE0F, p); /* U+FE0F is the Emoji variation selector */
       *p = '\0';
